@@ -1,8 +1,12 @@
+from matplotlib.pyplot import fill_between
+import matplotlib.pyplot as plt
 import numpy as np
 import astropy.constants as c
 import astropy.units as u
 from uncertainties.umath import  asin, sqrt, log, radians, sin, cos
 from .convert_param import P_to_aR, inclination
+from uncertainties import ufloat
+from scipy import interpolate
 			
 def planet_prot(f,req,mp,j2):
     """
@@ -61,55 +65,82 @@ def transit_prob(Rp, aR, e=0, w=90):
     return prob
     
     
-def ldtk_ldc(lambda_min,lambda_max,Teff,Teff_unc, logg,logg_unc,z,z_unc):
+def ldtk_ldc(Teff, logg, Z, filter, ld_law="qd", dataset="vis-lowres", nsamp=5000,unc_mult=10,nz=300,return_profile=False):
     """
-    Function to estimate quadratic limb darkening coefficients for a given star
+    Function to estimate different limb darkening coefficients for a given star using LDTk
     
     Parameters:
     ----------
-    lambda_min: Start wavelength of the bandpass filter.
-    
-    lambda_max: End  wavelength of the bandpass filter.
+    Teff : tuple
+        Effective temperature of the star.
 
-    Teff: Effective temperature of the star.
+    logg : tuple
+        Surface gravity of the star.
 
-    Teff_unc: Uncertainty in Teff.
+    Z : tuple
+        Metallicity of the star.
 
-    logg: Surface gravity of the star.
+    filter : filter transmission from SVO filter service(http://svo2.cab.inta-csic.es/theory/fps/)
+            should be an ldtk.filters.SVOFilter object or np.array of shape (2, N) such that wl = flt[0] \
+            and transmission= flt[1]
 
-    logg_unc: Uncertainty in logg.
+    ld_law : str
+        ld_law to calculate. must be one of ["ln","qd","tq","p2","p2mp","nl"]
 
-    z: Metallicity of the star.
+    dataset : str
+        one of 'vis', 'vis-lowres', 'visir', and 'visir-lowres'
 
-    z_unc: Uncertainty in z.
-    
+    nsamp : int
+        number of limb darkening profiles to generate
+
+    unc_mult : int
+        uncertainty multiplier. default is 10
+
+    nz : int
+        number of points for resampling from mu to z.  
+
+    return_profile : bool
+        whether to return ld profile rather than ld coefficients
 
     Returns
     -------
     cq, eq : Each an array giving the 2 quadractic limb darkening parameters and the errors associated with them 
 
+    ps : profile object returned if return_profile = True. access attributes such as: ps._mu, ps._mean, ps._std
     
     """
     
     from ldtk import LDPSetCreator, BoxcarFilter
+
+    assert ld_law in ["ln","qd","tq","p2","p2mp","nl"],f'ld_law must be one of ["ln","qd","tq","p2","p2mp","nl"] but {ld_law} given.' 
+    assert dataset in ['vis', 'vis-lowres', 'visir','visir-lowres'],f"dataset must be one of ['vis', 'vis-lowres', 'visir','visir-lowres'] but {dataset} given."
     
     # Define your passbands. Boxcar filters useful in transmission spectroscopy
-    filters = [BoxcarFilter('a', lambda_min, lambda_max)] 
+    # filters = [BoxcarFilter('a', lambda_min, lambda_max)] 
 
-    sc = LDPSetCreator(teff=(Teff,   Teff_unc),    # Define your star, and the code
-                   logg=(logg, logg_unc),    # downloads the uncached stellar
-                      z=(z, z_unc),    # spectra from the Husser et al.
-                     filters=filters)    # FTP server automatically.
+    sc = LDPSetCreator(teff=Teff, logg=logg, z=Z,    # spectra from the Husser et al.
+                        filters=filter, dataset=dataset)             # FTP server automatically.
 
-    ps = sc.create_profiles()                # Create the limb darkening profiles
-    cq,eq = ps.coeffs_qd(do_mc=True)         # Estimate quadratic law coefficients
+    ps = sc.create_profiles(nsamp)                # Create the limb darkening profiles\
+    if return_profile:
+        return ps
+    ps.set_uncertainty_multiplier(unc_mult)
+    ps.resample_linear_z(nz)
 
-    #lnlike = ps.lnlike_qd([[0.45,0.15],      # Calculate the quadratic law log
-#                       [0.35,0.10],      # likelihood for a set of coefficients
-#                       [0.25,0.05]])     # (returns the joint likelihood)
-
-    #lnlike = ps.lnlike_qd([0.25,0.05],flt=0) # Quad. law log L for the first filter
-    return cq, eq
+    #calculate ld profiles
+    if   ld_law == "qd":   c, e = ps.coeffs_qd(do_mc=True, n_mc_samples=100000,mc_burn=1000)         # Estimate quadratic law coefficients
+    elif ld_law == "p2":   c, e = ps.coeffs_p2(do_mc=True, n_mc_samples=100000,mc_burn=1000)
+    elif ld_law == "tq":   c, e = ps.coeffs_tq(do_mc=True, n_mc_samples=100000,mc_burn=1000)
+    elif ld_law == "p2mp": c, e = ps.coeffs_p2mp(do_mc=True, n_mc_samples=100000,mc_burn=1000)
+    elif ld_law == "nl":   c, e = ps.coeffs_nl(do_mc=True, n_mc_samples=100000,mc_burn=1000)
+    elif ld_law == "ln":   
+        c, e = ps.coeffs_ln(do_mc=True, n_mc_samples=100000,mc_burn=1000)
+        return ufloat(c,e)
+    
+    coeffs = []
+    for i in range(len(c[0])):
+        coeffs.append(ufloat(round(c[0][i],4), round(e[0][i],4)) )
+    return coeffs
     
 
 def sigma_CCF(res):
@@ -229,9 +260,9 @@ def ingress_duration(P, Rp, a, b=0, e=0, w=90, inc=None, total=True):
     
     return  (T14 - T23)/2.
 
-def T_eq(T_st,a_r):
+def T_eq(T_st,a_r, A_b =0 , f = 1/4):
     """
-    calculate equilibrium temperature of planet in Kelvin
+    calculate equilibrium/dayside temperature  of planet in Kelvin
     
     Parameters
     ----------
@@ -241,6 +272,12 @@ def T_eq(T_st,a_r):
         
     a_r: Array-like;
         Scaled semi-major axis of the planet orbit
+
+    A_b: Array-like;
+        Bond albedo pf the planet. default is zero
+
+    f: Array-like;
+        heat redistribution factor.default is 1/4 for uniform heat distribution and 2/3 for None. 
         
     Returns
     -------
@@ -249,7 +286,7 @@ def T_eq(T_st,a_r):
         Equilibrium temperature of the planet
     """
     print("T_st is {0:.2f}, a_r is {1:.2f}".format(T_st,a_r))
-    return T_st*sqrt(0.5/a_r)
+    return T_st*sqrt(1/a_r)* ((1-A_b)*f)**0.25
 
 
 def phase_fold(time, t0, P):
@@ -278,6 +315,48 @@ def phase_fold(time, t0, P):
     
     return ( (time-t0) % P) / float(P)
     
+def A_g(dF, Rp, aR):
+    """
+    Geometric albedo of a planet from occultation depth measurement
+
+    Parameters:
+    -----------
+    dF : occultation depth in ppm
+
+    Rp : planet-to-star radius ratio
+
+    aR : scaled semi-major axis
+
+    """
+    return (aR/Rp)**2 * dF*1e-6
+
+def albedo_temp_relation(Tst,Tpl,wl, L, aR, RpRs):
+    """
+    calculate the albedo of a planet as a function of dayside temperature from eclipse depth measurement 
+    
+    Parameters:
+    -----------
+    Tst: stellar surface temeprature in K
+    Tpl: planet dayside equilibrium temperature in K
+    L : eclipse depth in ppm
+    aR: scaled semi-major axis a/Rs
+    RpRs: planet-to-star radius ratio Rp/Rs
+    
+    Returns:
+    --------
+    Ag : planet albedo
+    """
+    from astropy import units as u
+    from astropy.modeling.models import BlackBody
+#     from astropy.visualization import quantity_support
+
+    bb = BlackBody(temperature=Tst*u.K)
+    wav = wl * u.AA
+    flux_st = bb(wav)
+    bb_p = BlackBody(temperature=Tpl*u.K)
+    flux_p = bb_p(wav)
+    ag = L*1e-6*(aR/RpRs)**2 - flux_p.value/flux_st.value * aR**2
+    return ag
 
 def msini(K,e,P,Mst, return_unit = "jupiter"):
 	
@@ -305,8 +384,10 @@ def msini(K,e,P,Mst, return_unit = "jupiter"):
     """
     Mj_sini  = 4.919*1e-3 *K * (1-e**2)**0.5 * P**(1/3) * Mst**(2/3)
     
-    if return_unit == "jupiter": return Mj_sini
-    if return_unit == "star": return Mj_sini*u.M_jup.to(u.M_sun)/Mst
+    if return_unit == "jupiter": 
+        return Mj_sini
+    if return_unit == "star": 
+        return Mj_sini*u.M_jup.to(u.M_sun)/Mst
     
 
 
@@ -344,3 +425,88 @@ def msini(K,e,P,Mst, return_unit = "jupiter"):
 #    return nu
     
 
+def PLATO_Noise_level(Vmag, timescale=60, prompt=False, show_plot=False):
+    """
+    calculate noise level expected from PLATO (using 24 cameras) for stars of different V magnitude.
+    The function performs a linear interpolation of the values in Table 2 of Samadi+2019 (https://doi.org/10.1051/0004-6361/201834822)
+
+    Parameters
+    ----------
+    Vmag : float
+        V magnitude of the star
+
+    timescale : float/int
+        timescale in minutes for which to calculate the noise level.
+        Default is 60 minutes as given in Samadi+2019
+
+    prompt : bool, optional
+        show prompt confirming if to extrapolate outside the Vmag value range (8.1 – 12.9 mag)of Samadi+2019 table
+
+    Returns
+    -------
+    float : 
+        noise level in ppm/timescale
+    """
+    if prompt:
+        extrapolate = 'n'
+        if 8.1 < Vmag > 12.9:
+            extrapolate = input("Vmag value is outside the tabulated range (8.1 to 12.9), do you want to extrapolate? y/n ")
+            assert extrapolate in ["y","n"], f"input must be 'y' or 'n' but {extrapolate} given."
+    else: extrapolate = 'y'
+
+    fill_value = "extrapolate" if extrapolate=="y" else np.nan
+    if 8.1 < Vmag > 12.9: print("... Extrapolating noise level outside Vmag range (8.1 - 12.9) ...")
+
+    V = np.array([8.1,8.5,9.0,9.5,10.0,10.5,11.0,11.5,12.0,12.5,12.9])
+    N_per_h = np.array([10.6, 12.9,16.4,20.8,26.7,34.5,44.8,59.2,79.1,106.8,138.5])
+
+    ip = interpolate.interp1d(V, N_per_h,fill_value=fill_value)
+
+    noise = ip(Vmag)*np.sqrt(60/timescale)
+
+    if show_plot:
+        plt.plot(V, N_per_h*np.sqrt(60/timescale),"bo")
+        v_grid = np.linspace( min([V[0],Vmag]), max([V[-1],Vmag]),50)
+        plt.plot(v_grid, ip(v_grid)*np.sqrt(60/timescale),"r--" )
+        plt.plot(Vmag, noise,"ro")
+        plt.xlabel("V mag", fontsize=14)
+        plt.ylabel("Noise [ppm]", fontsize=14)
+        plt.show()
+    return noise
+
+
+def Pdot(Mp_Mst,aR,Qst=1e6):
+    """
+    calculate period decay rate for a planet e.g https://iopscience.iop.org/article/10.3847/1538-3881/ab7374/pdf
+
+    Parameters
+    ----------
+
+    Mp_Mst : float
+        Mass ratio
+    aR : float
+        scaled semi-major axis
+    Qst : float
+        reduced tidal quality factor. norminal value 1e6
+    """
+
+    return -27*np.pi/(2*Qst) * Mp_Mst * 1/aR**5
+
+
+def inspiral_timescale(P,Mp_Mst,aR,Qst):
+    """
+    calculate timescale or englufment of a planet due to tides e.g https://iopscience.iop.org/article/10.3847/1538-3881/ab7374/pdf
+
+    Parameters
+    ----------
+    P : float
+        planet period
+    Mp_Mst : float
+        Mass ratio
+    aR : float
+        scaled semi-major axis
+    Qst : float
+        reduced tidal quality factor. norminal value 1e6
+    """
+    pdot = -27*np.pi/(2*Qst) * Mp_Mst * 1/aR**5
+    return P/pdot
\ No newline at end of file
